/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
/*
 * Copyright (c) 2009 ESRI All rights reserved under the copyright laws of the United States and applicable
 * international laws, treaties, and conventions. You may freely redistribute and use this sample code, with or without
 * modification, provided you include the original copyright notice and use restrictions. See use restrictions at
 * /arcgis/java/samples/userestrictions.
 */

package arcgissamples.geometry;

import java.io.*;

import com.esri.arcgis.datasourcesfile.*;
import com.esri.arcgis.geodatabase.*;
import com.esri.arcgis.geometry.*;
import com.esri.arcgis.system.*;

/**
 * The code in this class demonstrates how to modify the coordinates of a Polygon's geometry.
 */
public class ModifyPolygon
{	
	public ModifyPolygon()
	{
		
	}
	
	public static void main(String[] args)
	{
		System.out.println("Starting ModifyPolygon - An ArcObjects Java SDK Developer Sample");
		try
		{
			// Initialize the engine and licenses.
			EngineInitializer.initializeEngine();

			AoInitialize aoInit = new AoInitialize();
			initializeArcGISLicenses(aoInit);
			
			if (args.length != 1)
			{
				System.out.println("\nWrong number of arguments provided.\n" +
						"Correct usage is: ModifyPolygon [Source ShapeFile]\n" + 
						"[Source ShapeFile] - full path to polygon shapefile.\n");
				System.exit(0);
			}

			String inShape = args[0];

			String srcPath = inShape.substring(0, inShape.lastIndexOf(File.separator));
			String srcName = inShape.substring(inShape.lastIndexOf(File.separator) + 1);

			// Open a shapefile to get a polygon...
			ShapefileWorkspaceFactory wsFactory = new ShapefileWorkspaceFactory();
				
			IWorkspace ws = wsFactory.openFromFile(srcPath, 0);
			IFeatureWorkspace fws = (IFeatureWorkspace) ws;

			FeatureClass fc = new FeatureClass(fws.openFeatureClass(srcName));

			IFeature feature = fc.getFeature(1);

			// If the geometry type is not of type Polygon, exit...
			if (feature.getShape().getGeometryType() != esriGeometryType.esriGeometryPolygon)
			{
				System.out.println("The input shapefile must be of type Polygon.");
				aoInit.shutdown();
				System.exit(0);
			}

			Polygon poly = (Polygon) feature.getShape();

			ModifyPolygon modify = new ModifyPolygon();
			modify.modifyOneSegmentOfAPolygon(poly);
			modify.modifyOneVertexOfAPolygon(poly);
			aoInit.shutdown();

			System.out.println("Done");

			aoInit.shutdown();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}		

	/**
	 * This method moves the vertex of a Polygon
	 */
	private void modifyOneVertexOfAPolygon(Polygon polyColl) throws Exception
	{
		int[] lPart = new int[1];
		int[] lVertex = new int[1];
		boolean bright[] = new boolean[1];
		double[] dht = new double[1];

		// This could be an user input point (Ex: Mouse click etc.)
		double dr = 1;
		Point pHit = new Point(); // Create the output point
		polyColl.hitTest(polyColl.getFromPoint(), dr, esriGeometryHitPartType.esriGeometryPartVertex, pHit, dht, lPart,
				lVertex, bright);

		// Get the ring where the point is
		Ring pPathColl = (Ring) polyColl.getGeometry(lPart[0]);

		// Get the vertex that got hit
		Point pTrans2D = (Point) pPathColl.getPoint(lVertex[0]);

		// Move the point
		pTrans2D.move(-10, 0);

		// Update the point
		polyColl.updatePoint(lVertex[0], pTrans2D);

		// Let the polygon know that it's geometry has changed
		polyColl.geometriesChanged();

	}

	/**
	 * This method modifies the location of one of the line segments of a Polygon.
	 */
	private void modifyOneSegmentOfAPolygon(Polygon poly) throws Exception
	{
		int[] lPart = new int[1];
		int[] lSeg = new int[1];
		boolean bright[] = new boolean[1];
		double[] dht = new double[1];

		// Get the FROM point of the Polygon.
		Point pqpt = (Point) poly.getFromPoint();

		// This could be an user input point (Ex: Mouse click etc.)
		int dr = 1;
		Point pHit = new Point();
		poly.hitTest(pqpt, dr, esriGeometryHitPartType.esriGeometryPartBoundary, pHit, dht, lPart, lSeg, bright);

		// Get the ring where the point is
		Ring path = (Ring) poly.getGeometry(lPart[0]);

		// Get the segment that got hit
		Line pTrans2D = (Line) path.getSegment(lSeg[0]);

		pTrans2D.move(-10, 0);
		// In this case the segment doesn't have to be set back in the geometry, since the segment
		// was returned by reference by path.getSegment(lSeg[0])
		poly.geometriesChanged();

	}

	/**
	 * Initializes the lowest available ArcGIS License
	 */
	private static void initializeArcGISLicenses(AoInitialize aoInit)
	{
		try
		{
			if (aoInit.isProductCodeAvailable(esriLicenseProductCode.esriLicenseProductCodeEngine) == esriLicenseStatus.esriLicenseAvailable)
			{
				aoInit.initialize(esriLicenseProductCode.esriLicenseProductCodeEngine);
			}
			else if (aoInit.isProductCodeAvailable(esriLicenseProductCode.esriLicenseProductCodeArcView) == esriLicenseStatus.esriLicenseAvailable)
			{
				aoInit.initialize(esriLicenseProductCode.esriLicenseProductCodeArcView);
			}
			else
			{
				System.err.println("Could not initialize an Engine or ArcView license. Exiting application.");
				System.exit(-1);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
